program Beta;

{INTERFACE}

USES  Math,Crt,RuleSet, Global, OwlGraph,OwlAsm;

{PROCEDURE SetBetas;}


{IMPLEMENTATION       }

VAR
   ct : LongInt;
   outf : Text;
   Result : extended;
   u : extended;

Function RiskDeltaT(YrRisk : Single; DeltaT : Word):Extended;
         begin
              RiskDeltaT := 1 - XToTheY(1-YrRisk,1/DeltaT);
end;
PROCEDURE ScaleRisk;
          VAR
             i, Ct : Word;
          BEGIN
             FOR i := 0 to (Stages - 1) DO
               FOR Ct := 1 TO 8 DO
                   RiskArr[i,Ct] := RiskDeltaT(RiskArr[i,Ct],DeltaT);
END;
FUNCTION betacf(a,b,x: extended): extended;
LABEL 1;
CONST
   itmax=100;
   eps=3.0e-16;
VAR
   tem,qap,qam,qab,em,d,
   bz,bpp,bp,bm,az,app,
   am,aold,ap            : extended;
   m                     : word;
BEGIN
   qab := a+b;
   qap := a+1.0;
   qam := a-1.0;
   bz := 1.0-qab*x/qap;
   d := (b-1)*x/((qam+2)*(a+2));
   ap := 1.0+d;
   bp := bz+d;
   d := -(a+1.0)*(qab+1.0)*x/((a+2)*(qap+2));
   app := ap+d;
   bpp := 1/(bp+d*bz);
   am := ap*bpp;
   bm := bp*bpp;
   az := app*bpp;
   IF ((abs(az-1.0)) < (eps*abs(az))) THEN GOTO 1;
   FOR m := 2 to itmax DO BEGIN
      em := m;
      tem := em+em;
      d := em*(b-em)*x/((qam+tem)*(a+tem));
      ap := az+d*am;
      bp := 1+d*bm;
      d := -(a+em)*(qab+em)*x/((a+tem)*(qap+tem));
      app := ap+d*az;
      bpp := 1.0/(bp+d);
      aold := az;
      am := ap*bpp;
      bm := bp*bpp;
      az := app*bpp;
      IF ((abs(az-aold)) < (eps*abs(az))) THEN GOTO 1
   END;
1:   betacf := az;
END;

PROCEDURE Betai(VAR a,b,x,ibeta,fx: extended);
  VAR
    ch                      : CHAR;
    bt,compx,gamratio       : extended;
  BEGIN
    gamratio := LnGammaFn(a+b)-LnGammaFn(a)-LnGammaFn(b);
    IF ((x = 0.0) OR (x = 1.0)) THEN BEGIN
      bt := 0.0;
      fx := 0.0; END
    ELSE BEGIN
      bt := Exp(gamratio+a*Ln(x)+b*Ln(1.0-x));
      fx := Exp(gamratio+(a-1)*Ln(x)+(b-1)*Ln(1-x));
    END;
    IF (x < ((a+1.0)/(a+b+2.0))) THEN
      ibeta := bt*Betacf(a,b,x)/a
    ELSE BEGIN
      compx := 1-x;
      ibeta := 1.0-bt*Betacf(b,a,compx)/b;
    END;
  END;

PROCEDURE ComputeAB(Mu,Sigma : extended;VAR a,b : extended);

VAR
   MuVal : extended;
BEGIN
      Muval := (1-Mu)/Mu;
      a := (Muval/(Sqr(Sigma*(1+Muval)))-1)/(1+Muval);
      b := a*Muval;
END;

PROCEDURE Bisect(u, mu,sigma:extended; VAR x:extended);

CONST
   TOL = 1.0E-10;

VAR
    a, b, ULim, LLim, ibeta,fx : extended;

BEGIN
   IF (mu <> 0.0 )  AND (Sigma <> 0.0 ) then BEGIN
      ComputeAB(mu,sigma,a,b);
      LLim := 0.0;
      ULim := 1.0;
      REPEAT
         x := (LLim +ULim)/2.0;
         Betai(a,b,x,ibeta,fx);
         If (ibeta <= u) Then
            LLim := x
         ELSE ULim := x;
      UNTIL (ABS(ibeta-u)< TOL) OR (ABS(x - 1.0) < TOL) OR (ABS(x - 0.0) < TOL);  
   END
   ELSE IF (Sigma = 0.0) THEN
         x := mu
   ELSE IF (Mu = 0.0) THEN
      x := 0.0;
END;

PROCEDURE SetBetas;

CONST
    FirstTime    : Boolean  = TRUE;


VAR
   i, ct : Byte;
   FLedgeProb, u, Survival: EXTENDED;
   VarLimit : LimitType;
  

BEGIN
   Randomize;
   u  := rand;
   FOR i := 0 to (Stages-1) DO BEGIN
      IF FirstTime THEN
          SetVarLimits(VarLimit, i);
       FOR ct := 1 TO 8 DO BEGIN
         IF FirstTime THEN BEGIN
            IF (SurvivalVar[i,Ct] > VarLimit[1, Ct]) THEN BEGIN
               IF GraphOn THEN
                  RestoreTextMode;
               FatalErrors(8);
            END;
            IF (FledgeVar[i, Ct] > VarLimit[2, Ct]) THEN BEGIN
               IF GraphOn THEN
                  RestoreTextMode;
               FatalErrors(9);
            END;
            IF (Ct = 8) THEN
               FirstTime := FALSE;
         END;
         Bisect(u, SurvivalMean[i,ct], sqrt(SurvivalVar[i,ct]), Survival);
         RiskArr[i,ct] := 1-Survival;
         Bisect(u, FledgeMean[i,ct], sqrt(FledgeVar[i,ct]), FledgeProb);
         FledgeArr[i,ct] := FledgeProb;
       END;
   END;
 
   ScaleRisk;
END;
BEGIN
   Assign(Outf, 'betatest.dat');
   rewrite(outf);
   FOR ct := 1 TO 10000 DO BEGIN
      u := rand;
      Bisect(u, 0.3,0.2, result);
      Writeln(outf,result);
   end;
   close(outf);
END.