UNIT LineGra;

INTERFACE

USES Linlib,Graph,Math,Global,OwlGraph,Ruleset;

PROCEDURE CallGraph;

IMPLEMENTATION

TYPE
    ColorArr      = Array[1..3] OF Byte;
    StyleArr      = Array[1..3] OF Byte;
    TitleArr      = Array[1..3] OF String[20];
CONST
     OwlColor : ColorArr   = (Yellow,LightRed,White);
     OwlStyle : StyleArr   = (0,0,0);
     OwlTitle : TitleArr   = (' TOTAL ',' ADULT ',' JUVENILE ');
     GenTitle : TitleArr   = (' IDENTICAL ',' FRAC. LOST ','');
     GenColor : ColorArr   = (LightGreen,LightRed,White);
VAR
    Data          : RealDataArray;

PROCEDURE ScaleAxes(      N      : Word;
                    VAR Arr      : RealDataArray;
                    VAR IntArr   : DataArray;
                    VAR MaxVal   : Single;
                    VAR ScaleVal : Single;
                    ScaleTo      : Word);
          BEGIN
               IF N = 0 THEN Exit;
               MaxVal := Max(n,Arr);
               IF MaxVal > 0 THEN
                  ScaleVal := ScaleTo/MaxVal
               ELSE
                   ScaleVal := 0;
               RoundAndScaleRow(N,ScaleVal,Arr,IntArr);
END;
PROCEDURE DrawHashMarks(PosX,PosY,Number : Word;Vertical : Boolean);
          VAR
             St : String[2];
          BEGIN

               IF Vertical THEN BEGIN
                  Str(Number:2,St);
                  Line(PosX-3,PosY,PosX+3,PosY);
                  OutTextXY(PosX-12,PosY,St);
               END
               ELSE BEGIN
                    Str(Number,St);
                    Line(PosX,PosY + 3,PosX,PosY-3);
                    OutTextXY(PosX,PosY + 12,St);
               END;
END;
PROCEDURE SeparateExponent(N : Single;VAR Mant : Single;VAR Exp : ShortInt);
          VAR
             St : String[30];
             s  : Integer;
          BEGIN
               Str(N,St);
               Exp := Byte(St[Byte(St[0])]); {last letter}
               Dec(Exp,48);
               IF St[Length(St) - 4] = '-' THEN
                  Exp := -Exp;
               Val(Copy(St,1,5),Mant,s);     {3 decimal places}
               IF Mant < 2 THEN BEGIN
                     Dec(Exp);
                     Mant := Mant * 10;
               END;
END;
PROCEDURE CreateAxes(Time                       : Word;
                     VAR XTitle,YTitle          : String;
                     MaxY,ScaleX,ScaleY         : Single;
                     StartX,StartY              : Word);
          VAR
             XLabel,YLabel     : String[9];
             YExponent,
             XExponent         : ShortInt;
             XMantissa,
             YMantissa         : Single;
             StopX,StopY       : Integer;
             AxisScaleX,
             AxisScaleY        : Single;
             RoundUpX,RoundUpY : Byte;
             Ct                : Word;
          BEGIN
               YLabel := '';
               SeparateExponent(MaxY,YMantissa,YExponent);
               CASE YExponent OF
                    3 : YLabel := 'x 1000';
                    2 : YLabel := 'x 100';
                    1 : YLabel := 'x 10';
                   -1 : YLabel := 'x 0.1';
                   -2 : YLabel := 'x 0.01';
                   -3 : YLabel := 'x 0.001';
                   -4 : YLabel := 'x 0.0001';
               END;
               XLabel := '';
               SeparateExponent(Time,XMantissa,XExponent);
               CASE XExponent OF
                    3 : XLabel := 'x 1000';
                    2 : XLabel := 'x 100';
                    1 : XLabel := 'x 10';
                   -1 : XLabel := 'x 0.1'
               END;
               AxisScaleX := XToTheY(10,XExponent) * ScaleX;
               AxisScaleY := XToTheY(10,YExponent) * ScaleY;
               RoundUpX := Trunc(XMantissa + 0.5);
               RoundUpY := Trunc(YMantissa + 0.5);
               StopY := StartY - Round(RoundUpY * AxisScaleY);
               IF StopY < 10 THEN BEGIN
                  StopY := 10;
                  Dec(RoundUpY);
               END;
               StopX := StartX + Round(RoundUpX * AxisScaleX);
               IF StopX > GetMaxX THEN BEGIN
                  StopX := GetMaxX - 10;
                  Dec(RoundUpX);
               END;
               SetColor(LightCyan);
               SetLineStyle(0,0,1);
               Line(StartX,StartY,StartX,StopY);
               Line(StartX,StartY,StopX,StartY);
               SetTextJustify(1,1);
               OutTextXY(StartX - 12,StartY + 12,'0'); {Origin}
               FOR Ct := 1 TO RoundUpX DO
                   DrawHashMarks(StartX + Round(Ct * AxisScaleX),StartY,Ct,FALSE);
               FOR Ct := 1 TO RoundUpY DO
                   DrawHashMarks(StartX,StartY - Round(Ct * AxisScaleY),Ct,TRUE);
               SetColor(White);
               OutTextXY(GetMaxX SHR 1,GetMaxY-10,XTitle + ' ('+ XLabel + ')');
               SetTextStyle(0,1,1);
               OutTextXY(10,GetMaxY SHR 1,YTitle + ' (' + YLabel + ')');
               SetTextStyle(0,0,1);
END;
PROCEDURE GraphLegend(    N      : Byte;
                          Title  : String;
                      VAR Colors : ColorArr;
                      VAR Styles : StyleArr;
                      VAR Titles : TitleArr);
         VAR
            Ct : Byte;
         BEGIN
              MoveTo(150,10);
              SetTextStyle(0,0,1);
              SetTextJustify(0,1);
              SetColor(LightCyan);
              OutText(Title);
              FOR Ct := 1 TO N DO BEGIN
                  SetColor(White);
                  OutText(Titles[Ct]);
                  SetColor(Colors[Ct]);
                  SetLineStyle(Styles[Ct],0,3);
                  LineTo(GetX + 20,GetY);
              END;
END;
PROCEDURE DrawGraph(    Time          : Word;
                    VAR YVect         : RealDataArray;
                    XTitle,YTitle     : String;
                        LineOn        : Boolean;
                        Color         : Byte;
                        Style         : Byte;
                    VAR ScaleX,ScaleY : Single;
                    VAR Error         : Boolean);
          VAR
             IntYArr   : DataArray;
             GphX,GphY,
             Ct,DistX,
             DistY,
             StartX,StartY    : Word;
             MaxVal           : Single;
          BEGIN
               DistY := Round(0.75 * GetMaxY);
               DistX := Round(0.75 * GetMaxX);
               StartX := Round(GetMaxX * 0.1);
               StartY := Round(GetMaxY * 0.9);
               ScaleAxes(Time,YVect,IntYArr,MaxVal,ScaleY,DistY);
               ScaleX := DistX/Time;
               Error := ScaleX < 1;
               IF Error THEN
                  Exit;
               CreateAxes(Time,XTitle,YTitle,MaxVal,ScaleX,ScaleY,StartX,StartY);
               SetColor(Color);
               SetLineStyle(Style,0,3);
               MoveTo(StartX,StartY - IntYArr[1]);
               IF LineOn THEN
                  FOR Ct := 2 TO Time DO
                     LineTo(StartX + Round(Pred(Ct) * ScaleX),StartY - IntYArr[Ct])
               ELSE
                   FOR Ct := 2 TO Time DO
                    PutPixel(StartX + Round(Pred(Ct) * ScaleX),StartY - IntYArr[Ct],White);
END;
PROCEDURE AdditionalLines(    Time          : Word;
                          VAR YVect         : RealDataArray;
                              LineOn        : Boolean;
                              Color         : Byte;
                              Style         : Byte;
                              ScaleX,ScaleY : Single);
          VAR
             IntYArr   : DataArray;
             GphX,GphY,
             Ct,DistX,
             DistY,
             StartX,StartY    : Word;
             MaxVal,Dummy     : Single;
          BEGIN
               DistY := Round(0.75 * GetMaxY);
               StartX := Round(GetMaxX * 0.1);
               StartY := Round(GetMaxY * 0.9);
               RoundAndScaleRow(Time,ScaleY,YVect,IntYArr);
               SetColor(Color);
               SetLineStyle(Style,0,3);
               MoveTo(StartX,StartY - IntYArr[1]);
               IF LineOn THEN
                  FOR Ct := 2 TO Time DO
                     LineTo(StartX + Round(Pred(Ct) * ScaleX),StartY - IntYArr[Ct])
               ELSE
                   FOR Ct := 2 TO Time DO
                    PutPixel(StartX + Round(Pred(Ct) * ScaleX),StartY - IntYArr[Ct],White);
END;

PROCEDURE CallGraph;
          VAR
             Err               : Boolean;
             Ct,i              : Word;
             ScaleY,ScaleX     : Single;
             Ch,Ch2            : Char;
             temp : integer;
LABEL
     LOOP;
BEGIN
     IF Device = 0 THEN BEGIN
        InitGraph(Device,Mode,'');
        IF GraphResult <> 0 THEN
           FatalErrors(1);
        GraphOn := TRUE;
      END
      ELSE
          RestoreGraphics;
   LOOP:
      FOR Ct := 1 TO YrCtr DO  BEGIN
          Data[Ct] := 0;
             FOR i := 0 TO Stages-1 DO
                Data[Ct] := Data[Ct] + Totals[Ct]^[i];
      END;
     GraphLegend(3,'DEMOGRAPHICS ',OwlColor,OwlStyle,OwlTitle);
     DrawGraph(YrCtr,Data,'TIME IN YEARS','POPULATION',
     TRUE,OwlColor[1],OwlStyle[1],ScaleX,ScaleY,Err);
     FOR Ct := 1 TO YrCtr DO
         Data[Ct] := Data[Ct] - Totals[Ct]^[0];
     AdditionalLines(YrCtr,Data,TRUE,OwlColor[2],OwlStyle[2],ScaleX,ScaleY);
     FOR Ct := 1 TO YrCtr DO
         Data[Ct] := Totals[Ct]^[0];
     AdditionalLines(YrCtr,Data,TRUE,OwlColor[3],OwlStyle[3],ScaleX,ScaleY);
     GetKeys(Ch,Ch2);
     IF Ch = #27 THEN
        RestoreTextMode
     ELSE BEGIN
          ClearDevice;
          GraphLegend(1,'GENETICS ',GenColor,OwlStyle,GenTitle);
          DrawGraph(YrCtr,GenesLost,'TIME IN YEARS','GENETIC LOSS',
                    TRUE,GenColor[1],OwlStyle[1],ScaleX,ScaleY,Err);
          GetKeys(Ch,Ch2);
          IF Ch = #27 THEN
             RestoreTextMode
          ELSE BEGIN
               ClearDevice;
               GOTO Loop;
          END;
     END;



END;
BEGIN
END.






