
(*Owl allows the creation of a map of hexagonal cells with a variety of
  classification forms.  It then allows the simulation of owl demographics
  within this map.*)

(*Note: This program uses nasty tricks to obtain efficient memory management.
 it must, therefore, be compiled with range checking disabled {$R-} *)

{$R-,S-,V-,I-,A+,N+,E-,G-}
{$M 20000,0,655360}

USES Graph,Math,Crt,Scrlib,Dos,OwlAsm,Utility,LaserPr,ClassDef,Global;
CONST
     HabitatWidth  : Integer = 10;
     HabitatLength : Integer = 10;
     LandScapeClasses        = 6;        {0-5; actually 6 classes}
     OwlClasses              = 3;
     OwlTypeLetter : Array[0..5] OF Char = (' ','M','F','P',#0,#0);
     AltOwlTypeLetter : Array[0..5] OF Char = (' ','*','*','*',#0,#0);
     OwlTypeColor  : Array[0..5] OF Byte = (White,White,White,Yellow, 0,0);
     OwlTypeName   : Array[0..5] OF String[10] =
     ('No Owl','Male','Female','Pair','','');

     GraphOn : Boolean       = FALSE;
     Device  : Integer       = 0;
     Mode    : Integer       = 0;
     ZoomXStart : Integer    = 1;
     ZoomYStart : Integer    = 1;
     BlockOn    : Boolean    = FALSE;
     OwlOn      : Boolean    = FALSE;
     MoveFlag   : Boolean    = FALSE;
     OwlFlag                 = -8888;
     ClassFlag               = -9999;
     PointerColor            = LightRed;

{Colors}
     Video1 : Byte = 7;
     Video2 : Byte = 15;
     HiVid  : Byte = 15;
     RevVid : Byte = 112;
     RevVid2 : Byte = 112;
     Errorvid : Byte = 112;
     UnderVid : Byte = 1;
TYPE
    HabitatRow     = Array[1..10] OF Byte;
    HabitatRowPtr  = ^HabitatRow;
    HabitatArr     = Array[1..500] OF HabitatRowPtr;
    LandscapeClass = Array[1..LandscapeClasses] OF Integer;
    BitMap         = Array[1..10000] OF Byte;
    CharSet        = Set OF Char;
VAR
   Habitat              : HabitatArr;
   HexagonArr           : Array[1..7,1..2] OF Integer;
   HexSin30,HexCos30,
   HexRad               : Single;
   ZoomXStop,ZoomYStop  : Integer;

(****Error trapping procedures****)

PROCEDURE HaltProgram;
          BEGIN
               NormVideo;
               ClrScr;
               SetCursorSize(Cursor);
               Halt;
END;
PROCEDURE FatalErrors(Error : Byte);
BEGIN
     WriteLn;
     CASE Error OF
          1 : WriteLn('Error - BGI file not found on the default drive');
          2 : WriteLn('Error - Insufficient memory to run program');
          5 : WriteLn('Error - Problem with file Read/Write');
     END;
     Write('Hit <Enter> to exit program');
     ReadLn;
     HaltProgram;
END;

(****I/O procedures, TextMode****)

PROCEDURE SlowWrite(x,y : Byte; St : String);
          BEGIN
               GoToXY(x,y);
               Write(St);
END;
PROCEDURE CenterText(Y,Att : Byte; St : String);
          VAR
             Posit : Word;
          BEGIN
               Posit := Pred(Y) * 160 + (80 - (Byte(St[0]) DIV 2) * 2);
               VarFWrite(Posit,Att,St);
END;
PROCEDURE HighlightWrite(X,Y,N,Att,HiAtt : Byte;St : String);
          VAR
             Posit : Word;
          BEGIN
               Posit := GetScrPos(X,Y);
               FastWrite(Posit,Att,St);
               Inc(Posit,Pred(N)*2);
               WriteChar(Posit,St[N],HiAtt);
END;
PROCEDURE ClearLine(Y : Byte);
          VAR
             Posit : Word;
          BEGIN
               Posit := Pred(Y) * 160;
               CharToScreen(Posit,80,' ',TextAttr);
END;
PROCEDURE GetKeys(VAR Ch,ExtCh : Char);
           BEGIN
                ExtCh := #0;
                Ch := ReadKey;
                IF Ch = #0 THEN
                   ExtCh := ReadKey;
END;
FUNCTION ReturnYesNo : Char;
         VAR
            Ch,Ch2 : Char;
         BEGIN
              REPEAT
                    GetKeys(Ch,Ch2);
                    Ch := UpCase(Ch);
              UNTIL Ch IN ['Y','N'];
              ReturnYesNo := Ch;
END;
FUNCTION ReturnMenu(Chrs : CharSet) : Char;
         VAR
            Ch,Ch2 : Char;
         BEGIN
              REPEAT
                    GetKeys(Ch,Ch2);
                    Ch := UpCase(Ch);
              UNTIL Ch IN Chrs;
              ReturnMenu := Ch;
END;
PROCEDURE InputErrors(Low,High : Integer; ErrorNo : Byte);
          VAR
             Ch,Ch2      : Char;
             Temp1,Temp2 : String[20];
          BEGIN
               SetCursorSize(NoCursor);
               CASE ErrorNo OF
                   1 : CenterText(23,ErrorVid,'Error in Numeric Format');
                   2 : BEGIN
                           Str(Low,Temp1);
                           Str(High,Temp2);
                           CenterText(23,ErrorVid,'Error - Value out of Bounds');
                           CenterText(24,RevVid2,Temp1 + '  X  '+ Temp2);
                        END;
                    4 : CenterText(23,ErrorVid,'Error -  land classifications <> 100%');
                    5 : CenterText(23,ErrorVid,'Error in file read/write');
                    6 : CenterText(23,ErrorVid,'Error in file format');
               END;
               CenterText(25,RevVid,'Hit any Key to continue, <ESC> to quit');
               GetKeys(Ch,Ch2);
               IF Ch = #27 THEN HaltProgram;
               ClearLine(23);
               ClearLine(24);
               ClearLine(25);
               SetCursorSize(Cursor);
END;
FUNCTION InputInteger(     X,Y,Field : Byte;
                           Low,High  : Integer;
                       VAR Value     : Integer) : Boolean;
         VAR
              St     : String[20];
              Ch,Ch2 : Char;
              Err    : Integer;
              Holder : Integer;
         LABEL
              LOOP;
         BEGIN
              Holder := Value;
             LOOP:
              St := '                    ';
              GoToXY(X,Y);
              St[0] := Char(Field);
              Write(St);
              GoToXY(X,Y);
              St[0] := #0;
              REPEAT
                    GetKeys(Ch,Ch2);
                    IF (Ch IN ['0'..'9','-']) AND
                       (St[0] < Char(Field)) THEN BEGIN
                       St := St + Ch;
                       Write(Ch);
                    END;
                    IF (Ch = #8) AND (St[0] <> #0) THEN BEGIN
                          Dec(St[0]);
                          Write(Ch);
                          Write(#32);
                          Write(Ch);
                    END;
                    IF Ch = #27 THEN BEGIN
                       InputInteger := TRUE;
                       Exit;
                    END;
              UNTIL Ch = #13;
              IF St[0] = #0 THEN BEGIN
                  GoToXY(X,Y);
                  Write(Value:Field);
                  InputInteger := FALSE;
              END
              ELSE BEGIN
                   Val(St,Holder,Err);
                   IF Err <> 0 THEN BEGIN
                      InputErrors(Low,High,1);
                      GOTO LOOP;
                   END;
                   IF (Holder < Low) OR (Holder > High) THEN BEGIN
                      InputErrors(Low,High,2);
                      GOTO LOOP;
                   END;
                   Value        := Holder;
                   InputInteger := FALSE;
             END;
END;
FUNCTION InputText(     X,Y       : Byte;
                    VAR St        : String) : Boolean;
         VAR
              Ch,Ch2 : Char;
         LABEL
              LOOP;
         BEGIN
             LOOP:
              GoToXY(X,Y);
              St[0] := #0;
              REPEAT
                    GetKeys(Ch,Ch2);
                    IF NOT (Ch IN [#8,#27,#13]) THEN BEGIN
                       St := St + Ch;
                       Write(Ch);
                    END;
                    IF (Ch = #8) AND (St[0] <> #0) THEN BEGIN
                          Dec(St[0]);
                          Write(Ch);
                          Write(#32);
                          Write(Ch);
                    END;
                    IF Ch = #27 THEN BEGIN
                       InputText := TRUE;
                       Exit;
                    END;
              UNTIL Ch = #13;
              InputText := (St[0] = #0);
END;
(****Graphics I/O****)

{HercChange sets map to hercules standards in scale and color}

PROCEDURE HercChange;
          BEGIN
               ClassArray   := HercClassArray;
               ClassPattern := HercClassPattern;
               HexCos30 := HexCos30 * 1.6;
END;



PROCEDURE EraseArea(X1,Y1,X2,Y2:Word);
          BEGIN
               SetFillStyle(0,0);
               Bar(X1,Y1,X2,Y2);
END;
PROCEDURE GraphErrors(Low,High : Integer;ErrNo : Byte;VAR EscSw : Boolean);
          VAR
             Temp1,Temp2       : String[10];
             StartX,StartY     : Word;
             Ch,Ch2            : Char;
          BEGIN
               StartX := 140;
               StartY := 110;
               SetTextJustify(1,0);
               CASE ErrNo OF
                   1 : OutTextXY(StartX,StartY,'Bad numeric format');
                   2 : BEGIN
                           Str(Low,Temp1);
                           Str(High,Temp2);
                           OutTextXY(StartX,StartY,'Value out of bounds');
                           OutTextXY(StartX,StartY+10,Temp1 + '  X  '+ Temp2);
                        END;
                   3 : OutTextXY(StartX,StartY,'Invalid coordinates');
               END;
               OutTextXY(StartX,StartY+20,'<ESC> to quit');
               GetKeys(Ch,Ch2);
               EscSw := (Ch = #27);
               EraseArea(55,100,225,130);
END;
PROCEDURE SetGraphCursor(Y : Word);
          BEGIN
               OutTextXY(GetX,Y,'_');
END;
PROCEDURE KillGraphCursor(Y : Word);
          BEGIN
               SetColor(Black);
               OutTextXY(GetX,Y,'_');
               SetColor(White);
END;
FUNCTION InputGraphicInteger(     X,Y,Field : Word;
                                  Low,High  : Integer;
                              VAR Value     : Integer) : Boolean;
         VAR
              St          : String[20];
              Ch,Ch2      : Char;
              Err         : Integer;
              Holder      : Integer;
              TempX,TempY : Word;
              EscSw       : Boolean;
         LABEL
              LOOP;
         BEGIN
              TempX := x;
              TempY := y;
              Holder := Value;
              LOOP:
              SetTextJustify(0,0);
              X := TempX;
              Y := TempY;
              EraseArea(x,y-8,x+8*Field,y);
              St[0] := #0;
              MoveTo(x,y);
              REPEAT
                    SetGraphCursor(y);
                    GetKeys(Ch,Ch2);
                    KillGraphCursor(y);
                    IF (Ch IN ['0'..'9','-']) AND
                       (St[0] < Char(Field)) THEN BEGIN
                       St := St + Ch;
                       OutText(Ch);
                       SetGraphCursor(Y);
                    END;
                    IF (Ch = #8) AND (St[0] <> #0) THEN BEGIN
                          Dec(St[0]);
                          MoveTo(GetX-8,y);
                          Bar(GetX,y-8,GetX+8,y);
                          SetGraphCursor(Y);
                    END;
                    IF Ch = #27 THEN BEGIN
                       InputGraphicInteger := TRUE;
                       Exit;
                    END;
              UNTIL Ch = #13;
              IF St[0] = #0 THEN BEGIN
                  KillGraphCursor(Y);
                  Str(Value:Field,St);
                  OutText(St);
                  InputGraphicInteger := FALSE;
              END
              ELSE BEGIN
                   Val(St,Holder,Err);
                   IF Err <> 0 THEN BEGIN
                      GraphErrors(Low,High,1,EscSw);
                      IF EscSw THEN BEGIN
                         InputGraphicInteger := TRUE;
                         Exit;
                      END
                      ELSE
                          GOTO LOOP;
                   END;
                   IF (Holder < Low) OR (Holder > High) THEN BEGIN
                      GraphErrors(Low,High,2,EscSw);
                      IF EscSw THEN BEGIN
                         InputGraphicInteger := TRUE;
                         Exit;
                      END
                      ELSE
                          GOTO LOOP;
                   END;
                   Value        := Holder;
                   InputGraphicInteger := FALSE;
             END;
             KillGraphCursor(Y);
END;

(****Fundamental startup procedures****)

PROCEDURE InitializeGraphics;

          BEGIN
               InitGraph(Device,Mode,'');
               IF GraphResult <> 0 THEN
                  FatalErrors(1);
               GraphOn := TRUE;
END;
PROCEDURE RestoreTextMode;
          BEGIN
               ClearDevice;
               GraphOn := FALSE;
               RestoreCrtMode;
END;
PROCEDURE RestoreGraphics;
          BEGIN
               SetGraphMode(Mode);
               GraphOn := TRUE;
END;
PROCEDURE CreateHabitatArr(VAR Arr : HabitatArr);
          VAR
             Ct : Word;
          BEGIN
               IF MemAvail > LongInt(HabitatLength) * LongInt(HabitatWidth)
                  THEN BEGIN
                  FOR Ct := 1 TO HabitatLength DO BEGIN
                      GetMem(Arr[Ct],HabitatWidth);
                      ZeroFill(Arr[Ct]^,HabitatWidth);
                  END;
               END
               ELSE
                   FatalErrors(2);
END;

(****Graphics procedures associated with display of the habitat array****)

PROCEDURE DrawHexagon(StartX,StartY : Single;Class : Word);
          VAR
             Owl : Byte;
             Dim : Word;
          BEGIN
               IF (ZoomXStop - ZoomXStart) > (ZoomYStop - ZoomYStart) THEN
                  Dim := ZoomXStop - ZoomXStart
               ELSE
                  Dim := ZoomYStop - ZoomYStart;
               IF Dim < 50 THEN
                  SetColor(ExteriorColor)
               ELSE
                  SetColor(Black);
               HexagonArr[1,1] := Round(StartX + HexCos30);
               HexagonArr[1,2] := Round(StartY + HexSin30);
               HexagonArr[2,1] := HexagonArr[1,1];
               HexagonArr[2,2] := Round(StartY - HexSin30);
               HexagonArr[3,1] := Round(StartX);
               HexagonArr[3,2] := Round(StartY - HexRad);
               HexagonArr[4,1] := Round(StartX - HexCos30);
               HexagonArr[4,2] := HexagonArr[2,2];
               HexagonArr[5,1] := HexagonArr[4,1];
               HexagonArr[5,2] := HexagonArr[1,2];
               HexagonArr[6,1] := Round(StartX);
               HexagonArr[6,2] := Round(StartY + HexRad);
               HexagonArr[6,1] := Round(StartX);
               HexagonArr[6,2] := Round(StartY + HexRad);
               SetFillStyle(ClassPattern[GetClass(Class)],
                            ClassArray[GetClass(Class)]);
               FillPoly(6,HexagonArr);
               Owl := GetOwl(Class);
               IF Owl <> 0 THEN BEGIN
                  SetColor(OwlTypeColor[Owl]);
                  SetTextJustify(1,1);
                  IF HexRad < 3 THEN
                     PutPixel(Round(StartX), Round(StartY+1),
                         OwlTypeColor[Owl])
                  ELSE IF HexRad < 8 THEN
                  OutTextXY(Round(StartX),Round(StartY+1),
                                  AltOwlTypeLetter[Owl])
                  ELSE
                     OutTextXY(Round(StartX),Round(StartY+1),
                                     OwlTypeLetter[Owl]);
                  SetColor(ExteriorColor);
               END;
END;
PROCEDURE FindParams(VAR Radius : Single);
          VAR
             MaxX,MaxY                 : Word;
             ScaleX                    : Single;
             Scalar,DimX,DimY          : Word;
          BEGIN
               DimX := Succ(ZoomXStop-ZoomXStart);
               DimY := Succ(ZoomYStop-ZoomYStart);
               IF DimX > DimY THEN
                  Scalar := DimX
               ELSE
                   Scalar := DimY;
               MaxX := GetMaxX;
               MaxY := GetMaxY;
               IF Scalar > 10 THEN
                  ScaleX := 0.58 * MaxY/MaxX
               ELSE
                   ScaleX := 0.50 * MaxY/MaxX;
               Radius := (MaxX * ScaleX)/Scalar;
               HexSin30 := Sin(30 * DegToRad) * Radius;
               HexCos30 := Cos(30 * DegToRad) * Radius;
               HexRad := Radius;
END;
PROCEDURE WriteDimensions;
          VAR
             St1,St2    : String[4];
             Xloc,YLoc  : Word;
          BEGIN
               SetTextJustify(1,1);
               XLoc := GetMaxX - 70;
               YLoc := GetMaxY - 50;
               Rectangle(XLoc - 40,Yloc,XLoc+40,YLoc + 45);
               Str(ZoomXStart,St1);
               Str(ZoomYStart,St2);
               OutTextXY(XLoc,YLoc+12,'('+ St1 + ',' + St2+ ')');
               OutTextXY(XLoc,YLoc+24,'TO');
               Str(ZoomXStop,St1);
               Str(ZoomYStop,St2);
               OutTextXY(XLoc,YLoc+36,'('+ St1 + ','+ St2 + ')');
END;
PROCEDURE DisplayArray(VAR Arr : HabitatArr);
          VAR
             Ct1,Ct2                      : Word;
             Radius,StartX,
             StartY,Width                 : Single;
          BEGIN
               SetColor(ExteriorColor);
               FindParams(Radius);
               IF Device = HercMono THEN
                  HercChange;
               Width := HexCos30;
               StartY := 1;
               FOR Ct1 := ZoomYStart TO ZoomYStop DO BEGIN
                   StartY := StartY + 1.5 * Radius;
                   StartX := 1;
                   IF Odd(Ct1) THEN
                          StartX := StartX + Width;
                   FOR Ct2 := ZoomXStart TO ZoomXStop DO BEGIN
                       StartX := StartX + 2 * Width;
                       DrawHexagon(StartX,StartY,Arr[Ct1]^[Ct2]);
                   END;
               END;
               WriteDimensions;
               Rectangle(1,GetMaxY - 25,GetMaxX - 110,GetMaxY-5);
               Line(GetMaxX-110,1,GetMaxX-110,GetMaxY-50);
END;

(****
Legend, ChooseClass, PutPointer, and MovePointer are used to input
both owls and land classification types in order to to manipulate the
habitat map.
****)


PROCEDURE ChooseClass(VAR Class : Byte);
          VAR
             StartX        : Word;
             StartY        : Word;
             Ch,Ch2        : Char;
             Ct,Classes    : Byte;
          LABEL
               FIN;
          BEGIN
               if OwlOn then
                  Classes := Succ(OwlClasses)
               else
                   Classes := LandscapeClasses;
               EraseArea(2,GetMaxY-24,GetMaxX-111,GetMaxY-6);
               SetTextJustify(0,0);
               OutTextXY(100, GetMaxY - 10,
                         '<ENTER> to choose class, <ESC> to exit');
               SetTextStyle(0,0,2);
               SetTextJustify(0,1);
               StartX := GetMaxX - 35;
               Ct := Class;
               if OwlOn then
                  Inc(Ct);
               StartY := 30 * Ct + 15;
               SetColor(LightRed);
               OutTextXY(StartX,StartY,'<');
               REPEAT
                    GetKeys(Ch,Ch2);
                    IF Ch = #27 THEN
                       GOTO FIN;
                    CASE Ch2 OF
                         #72: BEGIN
                                   SetColor(Black);
                                   OutTextXY(StartX,StartY,'<');
                                   IF StartY <> 45 THEN
                                      Dec(StartY,30)
                                   ELSE
                                       StartY := Classes * 30 + 15;
                                   SetColor(LightRed);
                                   OutTextXY(StartX,StartY,'<');
                              END;
                         #80: BEGIN
                                   SetColor(Black);
                                   OutTextXY(StartX,StartY,'<');
                                   IF StartY <> Classes * 30 + 15 THEN
                                      Inc(StartY,30)
                                   ELSE
                                       StartY := 45;
                                   SetColor(LightRed);
                                   OutTextXY(StartX,StartY,'<');
                              END;
                    END;
              UNTIL (Ch = #13);

              Class := (StartY - 15) DIV 30;
              if OwlOn then
                 Dec(Class);
            FIN:
               SetColor(Black);
               OutTextXY(StartX,StartY,'<');
               SetColor(White);
               SetTextStyle(0,0,1);
               SetTextJustify(0,0);
               EraseArea(2,GetMaxY-24,GetMaxX-111,GetMaxY-6);
               OutTextXY(10, GetMaxY - 10,
           '<SPACE> alter array, <F1> change class, <F2> zoom, <Alt-E> exit ');
END;
PROCEDURE OpenWindow(StartX,StartY,EndX,EndY : Word; VAR Area : Bitmap);
          BEGIN
               IF ImageSize(StartX,StartY,EndX,EndY) >
                  SizeOf(Area) THEN BEGIN
                  CloseGraph;
                  HaltProgram;
               END;
               GetImage(StartX,StartY,EndX,EndY,Area);
               SetFillStyle(0,0);
               Bar3d(StartX,StartY,EndX,EndY,0,TRUE);
END;
PROCEDURE ZoomMenu(VAR EscSw : Boolean);
          VAR
             z                    : BitMap;
             StartX,StartY        : Word;
             NessMem              : Word;
          LABEL
               LOOP;
          BEGIN
               LOOP:
               StartX := 50;
               StartY := 50;
               SetTextJustify(0,0);
               OpenWindow(StartX,StartY,StartX + 178,StartY + 90,z);
               Inc(StartY,10);
               Inc(StartX,5);
               OutTextXY(StartX,StartY,'Start column :');
               OutTextXY(StartX,StartY+10,   'Start row    :');
               OutTextXY(StartX,StartY+20,'Stop  column :');
               OutTextXY(StartX,StartY+30,'Stop  row    :');
               StartX := 190;
               EscSw := InputGraphicInteger(StartX,StartY,3,1,
                                            HabitatWidth-4,ZoomXStart);
                IF EscSw THEN BEGIN
                   PutImage(50,50,z,CopyPut);
                   Exit;
               END;
               EscSw := InputGraphicInteger(StartX,StartY+10,3,1,
                                            HabitatLength-4,ZoomYStart);
                IF EscSw THEN BEGIN
                   PutImage(50,50,z,CopyPut);
                   Exit;
                END;
                EscSw := InputGraphicInteger(StartX,StartY+20,3,ZoomXStart+4,
                                            HabitatWidth,ZoomXStop);
                IF EscSw THEN BEGIN
                   PutImage(50,50,z,CopyPut);
                   Exit;
                END;
                EscSw := InputGraphicInteger(StartX,StartY+30,3,ZoomYStart+4,
                                            HabitatLength,ZoomYStop);
                IF EscSw THEN BEGIN

                   Exit;
                END;
                IF (ZoomXStart + 4 > ZoomXStop) OR (ZoomYStart + 4 > ZoomYStop) THEN
                   BEGIN
                        GraphErrors(1,1,3,EscSw);
                        IF EscSw THEN
                           PutImage(50,50,z,CopyPut)
                        ELSE
                            GOTO LOOP;
                END;

END;
PROCEDURE PutPointer(VAR PtrHolder;X,Y : Word;Class,Scale : Byte);
          VAR
             Pentagon        : Array[1..5,1..2] OF Word;
             ScaleX2,ScaleX  : Word;
             Owl             : Byte;
          BEGIN
               ScaleX2 := 2 * Scale;
               IF Device = HercMono THEN
                  ScaleX := Round(Scale * 1.5)
               ELSE
                   ScaleX := Scale;
               GetImage(X-ScaleX,Y,X+ScaleX,Y+ScaleX2,PtrHolder);
               Pentagon[1,1] := X;
               Pentagon[1,2] := Y;
               Pentagon[2,1] := X + ScaleX;
               Pentagon[2,2] := Y + Scale;
               Pentagon[3,1] := X + ScaleX;
               Pentagon[3,2] := Y + ScaleX2;
               Pentagon[4,1] := X - ScaleX;
               Pentagon[4,2] := Y + ScaleX2;
               Pentagon[5,1] := X - ScaleX;
               Pentagon[5,2] := Y + Scale;
               SetColor(PointerColor);
               IF OwlOn THEN BEGIN
                  Owl := GetOwl(Class);
                  IF Scale > 7 THEN BEGIN
                        SetFillStyle(1,Black);
                        FillPoly(5,Pentagon);
                        SetColor(OwlTypeColor[Owl]);
                        SetTextJustify(1,2);
                        OutTextXY(Round(X),Round(Y + Scale),
                                  OwlTypeLetter[Owl]);
                        SetColor(ExteriorColor);
                        SetTextJustify(0,0);
                     END
                     ELSE BEGIN
                          SetFillStyle(1,OwlTypeColor[Owl]);
                           FillPoly(5,Pentagon);
                     END;
                END
                ELSE BEGIN
                     SetFillStyle(ClassPattern[Class],
                                  ClassArray[Class]);
                     FillPoly(5,Pentagon);
                END;
                Setcolor(ExteriorColor);
END;


(****OwlLegend and Legend are here due to structural needs****)

(****Legend is used for exclusively for modifying land classification****)

PROCEDURE Legend;
          VAR
             Pos1,Pos2,YPos : Word;
             Ct             : Byte;
          BEGIN
               SetColor(White);
               SetTextJustify(1,1);
               Pos1 := GetMaxX - 60;
               Pos2 := Pos1 - 26;
               OutTextXY(Pos1,10,'LEGEND');
               YPos := 0;
               FOR Ct := 1 TO LandscapeClasses DO BEGIN
                   Inc(YPos,30);
                   OutTextXY(Pos1,Ypos,ClassNameArray[Ct]);
                   SetFillStyle(ClassPattern[Ct],ClassArray[Ct]);
                   Bar3d(Pos2,YPos+5,Pos2+50,YPos + 20,0,TRUE);
               END;
END;


(****OwlLegend is used exclusively for owl input****)

PROCEDURE OwlLegend;
          VAR
             Pos1,Pos2,YPos : Word;
             Ct             : Byte;
             Age            : String;
          BEGIN
               SetColor(White);
               SetTextJustify(1,1);
               Pos1 := GetMaxX - 60;
               Pos2 := Pos1 - 26;
               OutTextXY(Pos1,10,'LEGEND');
               YPos := 0;
               FOR Ct := 0 TO OwlClasses DO BEGIN
                   Inc(YPos,30);
                   OutTextXY(Pos1,Ypos,OwlTypeName[Ct]);
                   SetFillStyle(1,Black);
                   Bar3d(Pos2,YPos+5,Pos2+50,YPos + 20,0,TRUE);
                   SetColor(OwlTypeColor[Ct]);
                   OutTextXY(Pos1,YPos + 13,OwlTypeLetter[Ct]);
                   SetColor(White);
               END;
            
END;
Procedure ChangeCell(    Class                           : Byte;
                         StartX,StartY                   : Single;
                         Ct1,Ct2,Scale,ScaleX            : Word;
                     VAR Arr                             : HabitatArr;
                     VAR PtrHolder                        );
          Begin
               IF OwlOn THEN
                                AddOwl(Class,Arr[Ct1]^[Ct2])
                             ELSE
                                 ChangeClass(Class,Arr[Ct1]^[Ct2]);
                             PutImage(Round(StartX) - ScaleX,Round(StartY),
                                      PtrHolder,CopyPut);
                             DrawHexagon(StartX,StartY,Arr[Ct1]^[Ct2]);
                             PutPointer(PtrHolder,Round(StartX),
                                        Round(StartY),Class,Scale);
end;
PROCEDURE  MovePointer(Arr : HabitatArr;Draw : Boolean);
VAR
   PtrHolder        : Array[1..500] OF Byte;
   Width,Radius     : Single;
   StartY,StartX    : Single;
   Ct1,Ct2,Scale,
   ScaleX           : Word;
   Ch,Ch2           : Char;
   Class            : Byte;
   EscSw            : Boolean;
   St1,St2          : String[4];
LABEL
     LOOP;
BEGIN
     MoveFlag := TRUE;
     if OwlOn then
        Class := 0
     else
        Class := 1;
     LOOP:
     IF Draw THEN BEGIN
        DisplayArray(Habitat);
        IF OwlOn THEN
           OwlLegend
        ELSE
           Legend;
        Draw := FALSE;
     END;
     EraseArea(2,GetMaxY-24,GetMaxX-111,GetMaxY-6);
     SetTextJustify(0,0);
     OutTextXY(10, GetMaxY - 10,
              '<SPACE> alter array, <F1> change class, <F2> zoom, <Alt-E> exit ');
     FindParams(Radius);
     IF Device = HercMono THEN
        HercChange;
     Width := HexCos30;
     StartY := 1 + 1.5 * Radius;
     IF Odd(ZoomYStart) THEN
        StartX := 1 + 3 * Width
     ELSE
         StartX:= 1 + 2 * Width;
     Ct1 := ZoomYStart; Ct2 := ZoomXStart;
     Scale := Round(Width);
     IF Scale > 10 THEN
        Scale := 10;
     IF Device = HercMono THEN
        ScaleX := Round(Scale * 1.5)
     ELSE
         ScaleX := Scale;
     PutPointer(PtrHolder,Round(StartX),Round(StartY),Class,Scale);
     REPEAT
           Str(Ct2,St1);
           Str(Ct1,St2);
           IF Device <> HercMono THEN BEGIN
              SetFillStyle(1,15);
              Bar(GetMaxX-110,GetMaxY-70,GetMaxX-30,GetMaxY-50);
              SetColor(Blue);
           END
           ELSE BEGIN
               SetFillStyle(0,0);
               Bar(GetMaxX-110,GetMaxY-70,GetMaxX-30,GetMaxY-50);
               Rectangle(GetMaxX-110,GetMaxY-70,GetMaxX-30,GetMaxY-50);
           END;
           SetTextJustify(1,1);
           OutTextXY(GetMaxX - 70,GetMaxY - 59,'('+ St1 + ',' + St2 + ')');
           SetColor(White);
           GetKeys(Ch,Ch2);
           CASE Ch OF
                #32   : BEGIN
                            ChangeCell(Class,StartX,StartY,Ct1,Ct2,
                                       Scale,ScaleX,Arr,PtrHolder);
                        END;
                #0    : CASE Ch2 OF
                             #72 : BEGIN
                                        IF Ct1 <> ZoomYStart THEN BEGIN
                                           PutImage(Round(StartX)-ScaleX,
                                                    Round(StartY),
                                                      PtrHolder,CopyPut);
                                           Dec(Ct1);
                                           StartY := StartY - 1.5 * Radius;
                                           IF Odd(Ct1) THEN
                                              StartX := StartX + Width
                                           ELSE
                                               StartX := StartX - Width;
                                           PutPointer(PtrHolder,
                                                      Round(StartX),
                                                      Round(StartY),Class,
                                                      Scale);
                                        END;
                                   END;
                             #80 : BEGIN
                                        IF Ct1 <> ZoomYStop THEN BEGIN
                                           PutImage(Round(StartX)-
                                                    ScaleX,Round(StartY),
                                           PtrHolder,CopyPut);
                                           Inc(Ct1);
                                           StartY := StartY + 1.5 * Radius;
                                           IF Odd(Ct1) THEN
                                              StartX := StartX + Width
                                           ELSE
                                               StartX := StartX - Width;
                                                PutPointer(PtrHolder,
                                                      Round(StartX),
                                                      Round(StartY),
                                                      Class,Scale);
                                        END;
                                   END;
                        #75,#115 : BEGIN
                                        if Ch2 = #115 then
                                            ChangeCell(Class,StartX,StartY,Ct1,Ct2,
                                                      Scale,ScaleX,Arr,PtrHolder);
                                        IF Ct2 <> ZoomXStart THEN BEGIN
                                           PutImage(Round(StartX)-ScaleX,
                                                    Round(StartY),
                                                    PtrHolder,CopyPut);
                                           Dec(Ct2);
                                           StartX := StartX - 2 * Width;
                                           PutPointer(PtrHolder,
                                                      Round(StartX),
                                                      Round(StartY),Class,
                                                      Scale);
                                        END;
                                    END;
                         #77,#116 : BEGIN
                                         if Ch2 = #116 then
                                            ChangeCell(Class,StartX,StartY,Ct1,Ct2,
                                                      Scale,ScaleX,Arr,PtrHolder);
                                         IF Ct2 <> ZoomXStop THEN BEGIN
                                            PutImage(Round(StartX)-ScaleX,
                                                     Round(StartY),
                                                     PtrHolder,CopyPut);
                                            Inc(Ct2);
                                            StartX := StartX + 2 * Width;
                                            PutPointer(PtrHolder,
                                                       Round(StartX),
                                                       Round(StartY),
                                                       Class,Scale);
                                         END;
                                    END;
                               #71: BEGIN
                                         PutImage(Round(StartX)-ScaleX,
                                                  Round(StartY),PtrHolder,
                                                  CopyPut);
                                         StartY := 1 + 1.5 * Radius;
                                         IF Odd(ZoomYStart) THEN
                                            StartX := 1 + 3 * Width
                                         ELSE
                                             StartX := 1 + 2 * Width;
                                         Ct1 := ZoomYStart;
                                         Ct2 := ZoomXStart;
                                         PutPointer(PtrHolder,
                                                    Round(StartX),
                                                    Round(StartY),
                                                    Class,Scale);
                                    END;
                              #59 : BEGIN
                                         ChooseClass(Class);
                                         PutImage(Round(StartX)-ScaleX,
                                                  Round(StartY),PtrHolder,
                                                  CopyPut);
                                         PutPointer(PtrHolder,Round(StartX),
                                                    Round(StartY),Class,
                                                    Scale);
                                    END;
                              #60 : BEGIN
                                         ZoomMenu(EscSw);
                                         IF NOT EscSw THEN BEGIN
                                            ClearDevice;
                                            Draw := TRUE;
                                            GOTO LOOP;
                                         END;
                                    END;
                            #18   : BEGIN
                                         PutImage(Round(StartX) - Scale,
                                            Round(StartY),PtrHolder,CopyPut);
                                         Exit;
                        END;
                        END;
           END;
     UNTIL FALSE;
END;

(****GraphicDisplay is used if the array is going to be modified using
 MovePointer and ChooseClass immediately.  GraphicDisplay2 is used if
 there is doubt as to whether the user wants to manipulate the map****)

PROCEDURE GraphicDisplay(VAR Arr : HabitatArr;FirstTime,Initialize : Boolean);
          VAR
             Ct : Word;
          BEGIN
               IF Initialize THEN BEGIN
                  FOR Ct := 1 TO HabitatLength DO
                      FillChar(Arr[Ct]^,HabitatWidth,3 SHL 4);
               END;
               IF FirstTime THEN
                  InitializeGraphics
               ELSE
                   SetGraphMode(Mode);
               MovePointer(Habitat,TRUE);
END;
PROCEDURE GraphicDisplay2(FirstTime : Boolean);
          VAR
             Ch,Ch2 : Char;
             Err    : Boolean;
          LABEL
               Loop;
          BEGIN
               IF FirstTime THEN
                  InitializeGraphics
               ELSE
                   SetGraphMode(Mode);
               DisplayArray(Habitat);
               IF OwlOn THEN
                  OwlLegend
               ELSE
                   Legend;
               SetfillStyle(0,0);
               SetTextJustify(0,0);
               OutTextXY(50,GetMaxY - 10,
                         'Any key to continue, <F1> to modify, <Alt-P,L> to print');
               Loop:
               GetKeys(Ch,Ch2);
               if Ch2 = #38 then begin
                  LaserPrintScreen(Device,Err);
                  GOTO Loop;
               end;
               IF Ch2 = #59 THEN BEGIN
                  MovePointer(Habitat,FALSE);
                END;

END;

(**** SetRandomLandscape,InitializeLandClassification, SetUpMap
 and SetArraySize are used to manipulate land classifications exclusively
 and are not used in owl input procedures****)


PROCEDURE SetRandomLandscape(VAR TypeArr : LandscapeClass;
                             VAR Arr : HabitatArr);
          VAR
             Ct1,Ct2,Ct3,
             Sample         : Word;
             LandSum        : LandscapeClass;
             Ch,Ch2         : Char;
          BEGIN
               Randomize;
               LandSum[1] := TypeArr[1];
               FOR Ct1 := 2 TO LandScapeClasses DO
                   LandSum[Ct1] := LandSum[Pred(Ct1)] + TypeArr[Ct1];
               FOR Ct1:= 1 TO HabitatLength DO BEGIN
                   FOR Ct2 := 1 TO HabitatWidth DO BEGIN
                       Sample := Succ(Random(100));
                       Ct3 := 1;
                       WHILE Sample > LandSum[Ct3] DO
                             Inc(Ct3);
                       ChangeClass(Ct3,Habitat[Ct1]^[Ct2]);
                   END;
               END;
               Graphicdisplay2(TRUE);
END;
PROCEDURE InitializeLandClassification(VAR Arr : HabitatArr);
CONST
     LandClass : LandscapeClass = (100,0,0,0,0,0);
VAR
   Ch      : Char;
   Ct,Sum  : Word;
   LineCtr : Byte;
LABEL
     LOOP;
BEGIN
     ClrScr;
     SlowWrite(28,12,'Random location? (Y/N) : ');
     Ch := ReturnYesNo;
     IF Ch = 'N' THEN
        GraphicDisplay(Arr,TRUE,TRUE)
     ELSE BEGIN
          LOOP:
          ClrScr;
          LineCtr := 3;
          FOR Ct := 1 TO LandScapeClasses DO BEGIN
           IF LandScapeClasses > 8 THEN
              Inc(LineCtr)
           ELSE
               Inc(LineCtr,2);
           SlowWrite(5,LineCtr,'Percent in ' + ClassNameArray[Ct]);
           GoToXY(38,LineCtr);
           Write('(Default = ',LandClass[Ct]:3,')');
          END;
          LineCtr := 3;
          FOR Ct := 1 TO LandScapeClasses DO BEGIN
              IF LandScapeClasses > 7 THEN
                 Inc(LineCtr)
              ELSE
                  Inc(LineCtr,2);
           IF InputInteger(70,LineCtr,3,0,100,LandClass[Ct]) = TRUE THEN BEGIN
             InputErrors(1,1,3);
             GOTO Loop;
            END;
          END;
          Delay(100);
          Sum := 0;
          FOR Ct := 1 TO LandScapeClasses DO
              Sum := Sum + LandClass[Ct];
          IF Sum <> 100 THEN BEGIN
             InputErrors(1,1,4);
             GOTO LOOP;
          END;
          SetRandomLandScape(LandClass,Habitat);
     END;
END;
PROCEDURE SetArraySize;
          LABEL
               LOOP;
          BEGIN
               LOOP:
               ClrScr;
               SlowWrite(5,10,'The width of the habitat  (default = 10) : ');
               SlowWrite(5,12,'The length of the habitat (default = 10) : ');
               IF InputInteger(70,10,3,5,500,HabitatWidth) THEN BEGIN
                  InputErrors(1,1,3);
                  GOTO LOOP;
               END;
               IF InputInteger(70,12,3,5,500,HabitatLength) THEN BEGIN
                  InputErrors(1,1,3);
                  GOTO LOOP;
               END;
               Delay(100);
END;
PROCEDURE SetUpMap;
          BEGIN
               TextAttr := Video1;
               SetArraySize;
               ZoomXStop  := HabitatWidth;
               ZoomYStop  := HabitatLength;
               CreateHabitatArr(Habitat);
               InitializeLandClassification(Habitat);
               RestoreTextMode;
END;

(****SaveToFile is used for all file saves, RetrieveFile for all
 file input****)

PROCEDURE SaveToFile(Flag : Integer;OwlFile,Menu : Boolean);
          VAR
             OutF     : Text;
             Ch,Ch2   : Char;
             Ct1,Ct2  : Word;
             FileName : String;
          LABEL
               LOOP;
          BEGIN
               LOOP:
               ClrScr;
               IF Menu THEN BEGIN
                  SlowWrite(21,12,'Save map as a file? (Y/N) : ');
                  Ch := ReturnYesNo;
               END
               ELSE
                   Ch := 'Y';
               IF Ch = 'Y' THEN BEGIN
                  SlowWrite(21,14,'The file name : ');
                  ReadLn(FileName);
                  Assign(OutF,FileName);
                  ReSet(OutF);
                  IF IOResult = 0 THEN BEGIN
                     CenterText(23,ErrorVid,'The file already exists.');
                     CenterText(24,RevVid,
                                'Hit <ESC> to abort, any key to continue');
                     GetKeys(Ch,Ch2);
                     ClearLine(23);
                     ClearLine(24);
                     IF Ch = #27 THEN BEGIN
                        InputErrors(1,1,3);
                        GOTO LOOP;
                     END;
                  END;
                  ReWrite(OutF);
                  IF IOResult <> 0 THEN BEGIN
                     InputErrors(1,1,5);
                     GOTO LOOP;
                  END
                  ELSE BEGIN
                       WriteLn(OutF,Flag);
                       WriteLn(OutF,HabitatLength,' ',HabitatWidth);
                       FOR Ct1 := 1 TO HabitatLength DO
                           FOR Ct2 := 1 TO HabitatWidth DO BEGIN
                               IF OwlFile THEN
                                  Write(Outf,GetOwl(Habitat[Ct1]^[Ct2]),' ')
                               ELSE
                                  Write(Outf,Habitat[Ct1]^[Ct2],' ');
                               IF IOResult <> 0 THEN BEGIN
                                  Close(Outf);
                                  InputErrors(1,1,5);
                                  GOTO Loop;
                               END;
                           END;
                  END;
                  Close(Outf);
               END;
END;

PROCEDURE InputFileName(    StartX,StartY,BackColor,
                            HiColor                     : Byte;
                            St                          : String;
                        VAR Inf                         : Text;
                        VAR EscSw                       : Boolean);
          VAR
             FileName : String;
             S        : SearchRec;
             Posit,Ct : Word;
             AttHold  : Byte;
          LABEL
               Loop;
          BEGIN
               AttHold := TextAttr;
               TextAttr := BackColor;
               LOOP:
               SetCursorSize(NoCursor);
               ClrScr;
               Ct    := 0;
               Posit := 10;
               FindFirst('*.*',Archive,S);
               WHILE (DosError = 0) AND (Ct <= 60) DO BEGIN
                    VarFWrite(Posit,HiColor,S.Name);
                    FindNext(S);
                   Inc(Posit,40);
                    Inc(Ct);
                END;
                SlowWrite(StartX,StartY,St);
                SetCursorSize(Cursor);
                EscSw := InputText(StartX + Byte(St[0]),StartY,FileName);
                IF EscSw THEN
                   Exit;
                Assign(Inf,Filename);
                Reset(Inf);
                IF IOResult <> 0 THEN BEGIN
                   InputErrors(1,1,5);
                   GOTO LOOP;
                END;
                TextAttr := AttHold;
END;
PROCEDURE RetrieveFile(HiColor    : Byte;
                       FlagNo     : Integer;
                       FirstTime  : Boolean;
                   VAR EscSw      : Boolean);
          VAR
             Ct1,Ct2,
             Posit    : Word;
             Flag     : Integer;
             S        : SearchRec;
             FileName : String;
             Inf      : Text;
             X        : Byte;
          LABEL
               LOOP;
          BEGIN
               LOOP:
                InputFileName(10,22,TextAttr,HiColor,'The map file : ',Inf,EscSw);
                IF EscSw THEN
                   Exit;
                ReadLn(Inf,Flag);
                IF Flag <> FlagNo THEN BEGIN
                   InputErrors(1,1,6);
                   GOTO LOOP;
                END;
                Readln(Inf,HabitatLength,HabitatWidth);
                ZoomXStop  := HabitatWidth;
                ZoomYStop  := HabitatLength;
                IF FirstTime THEN
                   CreateHabitatArr(Habitat);
                FOR Ct1 := 1 TO HabitatLength DO
                    FOR Ct2 := 1 TO HabitatWidth DO BEGIN
                        Read(Inf,X);
                        IF FlagNo = OwlFlag THEN
                           AddOwl(X,Habitat[Ct1]^[Ct2])
                        ELSE
                            Habitat[Ct1]^[Ct2] := X;
                    END;
                Close(Inf);
END;

(****Basic calling procedures and menus****)

PROCEDURE GetGISOutput(VAR Arr : HabitatArr;VAR GISSw : Boolean);
          VAR
             Inf     : Text;
             EscSw   : Boolean;
             X,Y,LC  : Integer;
             Num     : LongInt;
             Marker  : Pointer;
             Ct1,Ct2 : Word;
             Percent,
             PolyNum : Single;
             CellNo  : LongInt;
          BEGIN
               GISSw := FALSE;
               ClrScr;
               InputFileName(20,22,30,31,'The GIS output file : ',
                             Inf,EscSw);
               IF EscSw THEN Exit;
               ClrScr;
               X := 1;
               Y := 1;
               SlowWrite(1,11,'The Number of Cells (E/W) : ');
               SlowWrite(1,13,'The Number of Cells (N/S) : ');
               SlowWrite(1,15,'The Off-Map Land Class    : ');
               EscSw := InputInteger(70,11,5,1,1000,X);
               IF EscSw THEN Exit;
               EscSw := InputInteger(70,13,5,1,1000,Y);
               IF EscSw THEN Exit;
               EscSw := InputInteger(70,15,5,1,LandscapeClasses,LC);
               IF EscSw THEN Exit;
               HabitatWidth  := X;
               HabitatLength := Y;
               ZoomXStop  := HabitatWidth;
               ZoomYStop  := HabitatLength;
               CellNo := LongInt(HabitatWidth) * LongInt(HabitatLength);
               Mark(Marker);
               CreateHabitatArr(Arr);
               FOR Ct1 := 1 TO Y DO BEGIN
                   FOR Ct2 := 1 TO X DO BEGIN
                       ChangeClass(LC,Arr[Ct1]^[Ct2]);
                   end;
               end;
               While not EOF(Inf) do begin
                     ReadLn(Inf,PolyNum,Percent);
                     if PolyNum > CellNo then
                        FatalErrors(5);
                        Num := Round(PolyNum);
                        Ct2 := Num mod X;
                        if Ct2 = 0 then
                           Ct2 := X;
                        Ct1 := Succ(Pred(Num) div X);
                        IF IOResult <> 0 THEN BEGIN
                           Release(Marker);
                           Close(Inf);
                           InputErrors(1,1,5);
                           Exit;
                        END;
                        IF Percent <= 0.25 THEN
                           ChangeClass(6,Arr[Ct1]^[Ct2])
                        ELSE
                        IF Percent <= 0.50 THEN
                           ChangeClass(5,Arr[Ct1]^[Ct2])
                        ELSE
                        IF Percent <= 0.75 THEN
                           ChangeClass(4,Arr[Ct1]^[Ct2])
                        ELSE
                        IF Percent <= 1.0 THEN
                           ChangeClass(3,Arr[Ct1]^[Ct2])
                        ELSE
                            ChangeClass(1,Arr[Ct1]^[Ct2]);
               END;
               GISSw := TRUE;
END;
FUNCTION ChooseKeyboardOrFile(PassedCh  : Char;
                              Flag      : Integer;
                              FirstTime : Boolean):Boolean;
          VAR
             EscSw,GISChosen,AFile : Boolean;
          BEGIN
               EscSw := FALSE;
               GisChosen := False;
               AFile := (PassedCh In ['M','G','F']);
               ChooseKeyboardOrFile := AFile;
               if AFile then begin
                  if PassedCh = 'G' then begin
                     if Not OwlOn then
                            GetGISOutput(Habitat,GisChosen);
                  end;
                  if Not GISChosen then begin
                     IF OwlOn THEN
                        RetrieveFile(31,OwlFlag,FirstTime,EscSw)
                     ELSE
                         RetrieveFile(31,Flag,FirstTime,EscSw);
                     IF EscSw THEN ChooseKeyboardOrFile := FALSE;
                  END;
               end;
END;

PROCEDURE InputClass(Ch : Char);
          BEGIN
               IF ChooseKeyboardorFile(Ch,ClassFlag,TRUE) THEN BEGIN
                    GraphicDisplay2(TRUE);
                    RestoreTextMode;
                    IF MoveFlag THEN
                     SaveToFile(ClassFlag,FALSE,TRUE);
               END
               ELSE BEGIN
                    SetupMap;
                    SaveToFile(ClassFlag,FALSE,TRUE);
               END;
END;
PROCEDURE InputOwls(PassedCh : Char);
          VAR
             Flag   : Boolean;
          BEGIN
               OwlOn := TRUE;
               IF ChooseKeyboardOrFile(PassedCh,OwlFlag,FALSE) THEN
                     GraphicDisplay2(FALSE)
                  ELSE
                     GraphicDisplay(Habitat,FALSE,FALSE);
                  RestoreTextMode;

END;
PROCEDURE OwlSimulationMenu;
          VAR
             Ch : Char;
          BEGIN
               REPEAT
                    SetCursorSize(NoCursor);
                    ClrScr;
                    CenterText(5,Video2,
                    'Save owls separately or create simulation matrix');
                    HighlightWrite(30,7,6,Video1,HiVid,'save Owl file');
                    HighlightWrite(30,8,6,Video1,HiVid,'save Simulation file');
                    HighlightWrite(30,9,1,Video1,HiVid,'Quit');
                    Ch := ReturnMenu(['O','S','Q',#27] );
                    SetCursorSize(Cursor);
                    CASE Ch OF
                         'O' : SaveToFile(OwlFlag,TRUE,FALSE);
                         'S' : SaveToFile(ClassFlag,FALSE,FALSE);
                     'Q',#27 : Exit;
                    END;
              UNTIL FALSE;
END;
PROCEDURE FileInputMenu;
          VAR
             Ch : Char;
          BEGIN
               SetCursorSize(NoCursor);
               ClrScr;
               CenterText(5,Video2,
          'Input from the keyboard, a standard Map file or a GIS output file');
               HighlightWrite(30,7,1,Video1,HiVid,'Keyboard');
               HighlightWrite(30,8,1,Video1,HiVid,'Map File');
               HighlightWrite(30,9,1,Video1,HiVid,'GIS file');
               HighlightWrite(30,10,1,Video1,HiVid,'Quit');
               Ch := ReturnMenu(['K','M','G','Q',#27] );
               SetCursorSize(Cursor);
               if Ch in ['Q',#27] then
                  HaltProgram
               else
                   InputClass(Ch);
END;
PROCEDURE OwlInputMenu;
          VAR
             Ch : Char;
          BEGIN
               SetCursorSize(NoCursor);
               ClrScr;
               CenterText(5,Video2,
          'Input owls from the keyboard, or from a file');
               HighlightWrite(30,7,1,Video1,HiVid,'Keyboard');
               HighlightWrite(30,8,1,Video1,HiVid,'File');
               HighlightWrite(30,9,1,Video1,HiVid,'Quit');
               Ch := ReturnMenu(['K','F','Q',#27] );
               SetCursorSize(Cursor);
               if Ch in ['Q',#27] then
                  HaltProgram
               else
                   InputOwls(Ch);
END;

PROCEDURE SetColors;
          BEGIN
               IF Color THEN BEGIN
                  Video1     := 30;
                  Video2     := 31;
                  HiVid      := 28;
                  RevVid     := 112;
                  RevVid2    := 63;
                  ErrorVid   := 79;
                  UnderVid   := 28;
                  TextAttr   := Video1;
               END;
END;

(**********MAIN PROGRAM***********)

BEGIN
     SetColors;
     FileInputMenu;
     OwlInputMenu;
     OwlSimulationMenu;
     HaltProgram;
END.

