UNIT RuleSet;

INTERFACE

USES Global,math,Owlasm,Utility;
Const
   NumRules = 11;

TYPE
    
    RealCriteria   = Array[1..8] OF single;
    AcceptCriteria = RealCriteria;
    RealMatrix     = Array[0..9] OF RealCriteria;
    RuleType  = Array[1..NumRules] OF Single;
      LimitType = Array[1..2, 1..8] OF Real;
    SigmaArrayType = Array[1..8] OF Single;
CONST


   JSurvivalMean      : RealCriteria   = (0, 0, 0.29, 0.29, 0.25, 0.21, 0, 0);
   SASurvivalMean     : RealCriteria   = (0, 0, 0.90, 0.90, 0.90, 0.83, 0, 0);
   ASurvivalMean      : RealCriteria   = (0, 0, 0.92, 0.91, 0.90, 0.83, 0, 0);
   JSurvivalVar       : RealCriteria   = (0, 0, 0.00, 0.00, 0.00, 0.00, 0, 0);
   SASurvivalVar      : RealCriteria   = (0, 0, 0.00, 0.00, 0.00, 0.00, 0, 0);
   ASurvivalVar       : RealCriteria   = (0, 0, 0.00, 0.00, 0.00, 0.00, 0, 0);
   JFledgeMean        : RealCriteria   = (0, 0, 0, 0, 0, 0, 0, 0);
   SubFledgeMean      : RealCriteria   = (0,0,  0.0,0.0,0,0,0,0);
   AFledgeMean        : RealCriteria   = (0,0,  0.34,0.33,0.27,0.26,0,0);
   JFledgeVar         : RealCriteria   = (0, 0, 0, 0, 0, 0, 0, 0);
   SubFledgeVar       : RealCriteria   = (0,0, 0.00,0.00,0,0,0,0);
   AFledgeVar         : RealCriteria   = (0,0, 0.00,0.00,0,0,0,0);
   FledgeNo           : Byte = 2;
   MaleFledgeProb     : Single = 0.5;  {Proportion of males}
   MaxStage           : Byte   = 2;

   LandClassOk        : AcceptCriteria = (0,0,1,1,1,1,1,1);  {Ok for nesting}
   Aversion           : RealCriteria = (0,0,1,0.9,0.7,0.5,0.0,0.0);
   FemSearchProb      : Single = 0.5;       {Probability that a female will find a male}
                            {in an adjacent cell}
   PairAversion       : Single = 0.5;       {male aversion to entering occupied territory.}
                            {a weighting factor, 0 = total aversion}
   DirectionalWeight  : Single = 2.0;   {tendency to go in the same direction}
   WanderLust         : RealCriteria = (0,0,0.5,0.5,0.5,0.5,0.0,0.0);
                                    { -> 0 = increasing wander lust}
   BoundaryCondition  : Byte = 3;   {1 = reflecting, 2 = absorbing, and 3 = wrap}
   Stages             : Word = 3;
   Runs               : Word  = 1;
   Years              : Word  = 100;
   DeltaT             : Word  = 20;   {10 = 1 tenth of a year, 20 = 1/20th}
 

VAR
   survivalmean, SurvivalVar, FledgeMean, FledgeVar : RealMatrix;
   FledgeArr, RiskArr                               : RealMatrix;
   cumfledgeprob                                    : Single;
   Range,Rule                                       : RuleType;
   SigmaArray                                       : SigmaArrayType;

Procedure SetDefaultRules;
Procedure SetVarLimits(VAR VarLimit : LimitType; StageNo : Word);
IMPLEMENTATION



Procedure SetDefaultRules;

VAR
   ct : Byte;

BEGIN
   Stages       := 3;
   ZeroFill(survivalmean,SizeOf(SurvivalMean));
   ZeroFill(SurvivalVar, SizeOf(SurvivalVar));
   ZeroFill(FledgeMean, SizeOf(FledgeMean));
   ZeroFill(FledgeVar, SizeOf(FledgeVar));
   survivalmean[0] := JSurvivalMean;
   survivalmean[1] := SASurvivalMean;
   survivalmean[2] := ASurvivalMean;
   SurvivalVar[0]   := JSurvivalVar;
   SurvivalVar[1]   := SASurvivalVar;
   SurvivalVar[2]   := ASurvivalVar;
   FledgeMean[0] := JFledgeMean;
   FledgeMean[1] := SubFledgeMean;
   FledgeMean[2] := AFledgeMean;
   FledgeVar[0]  := JFledgeVar;
   FledgeVar[1] := SubFledgeVar;
   FledgeVar[2] := AFledgeVar;
End;


PROCEDURE SetVarLimits(VAR VarLimit : LimitType; StageNo : Word);

VAR
   ct : Integer;
   Mean : Real;

BEGIN
      FOR ct := 1 TO 8 DO BEGIN
          Mean := SurvivalMean[StageNo, ct];
          IF (Mean < 0.011) OR (Mean > 0.989) THEN
             VarLimit[1,Ct] := 0
          ELSE
             VarLimit[1, ct] := Mean - sqr(Mean)-0.01;
          Mean := FledgeMean[StageNo, ct];
          IF (Mean < 0.011) OR (Mean > 0.989) THEN
             VarLimit[2,Ct] := 0
          ELSE
             VarLimit[2, ct] := Mean - sqr(Mean)-0.01;
      END;
END;

BEGIN
END.
