Unit SumStat;

Interface

uses crt,linlib,global,utility,scrlib, RuleSet, Graph, OwlGraph;

Procedure ProduceSummaryStats(    Runs,Years           : Word;
                              VAR TotalPop             : RealDataArray;
                                  ExtinctionTime       : Single;
                                  Extinctions          : Word;
                                  MinPop               : Word;
                                  MeanDistDeath,
                                  MeanDistPair         : Single;
                              VAR OutFil               : Text;
                                  PrinterOn            : Boolean);
Procedure CreateSumStatsFile(MeanTimeExt,MeanDeathDist,
                             MeanPairDist                         : Single;
                             MinimumPop,ExtNo                     : Word;
                             FileName                             : String);
Procedure ReadSumStatsFile(VAR MeanTimeExt,MeanDeathDist,
                               MeanPairDist                     : Single;
                           VAR MinimumPop,ExtNo                 : Word;
                               FileName                         : String);

Function ModifyFileName(Filename : String;Suffix :String4) : String;
PROCEDURE OutPutToScreen(VAR Inf : Text; FN : String);

Implementation

Type
    SumStatRec = Record
                       MTExtinct,AvgDeath,AvgPair : Single;
                       MinPop,ExtNo               : Word;
                 end;
    TitleArr      = Array[1..7] OF String[40];
var
   MeanExtTime,MeanDeathDist,MeanPairDist    : Single;
      GlobRuns,Years,MinPop,ExtinctNo           : Word;
    Pop,Pair,
   PopSD,PairSD,
   IdentByDescentSD,
   IdentByDescent                            : DataPtr;
{Replaces the suffix of a filename with a specific suffix.}

Function ModifyFileName(Filename : String;Suffix : String4) : String;
         VAR
            PeriodLoc : Byte;
         begin
              PeriodLoc := LastCharPos('.',FileName);
              if PeriodLoc <> 0 then
                 FileName := Copy(FileName,1,Pred(PeriodLoc));
              if Suffix[1] <> '.' then     {make sure a period is present}
                 Suffix := '.' + Suffix;
              ModifyFileName := FileName + Suffix;
end;

{given 2 vectors, LeastSquare computes the slope and intercept. Heap
 requirements = 8 * n}

PROCEDURE LeastSquare(   n                     : Word;
                     VAR X,Y                   : RealDataArray;
                     VAR a,b                   : Single;{a=intercept,b=slope}
                     VAR Error                 : Byte);

{Error : 0 = all's well; 1 = infinite slope;
         3 = not enough heap}

VAR
   MeanX,MeanY,
   XY,XSq              : Extended;          {hold the temporaries as extended}
   Temp1               : RealDataArray;
   nSpace              : Word;
BEGIN
     Error := 0;

{first compute the line}

     MoveRow(X,Temp1,n);                                             {save X}
     MeanY := Mean(n,Y);                   {get mean of Y, negative mean of X}
     MeanX := -Mean(n,X);
     AddConst(n,MeanX,Temp1);                   {create vector X - meanX = x}
     Xsq := VectxVect(Temp1,Temp1,n);               {create dot product x^2}
     XY := VectxVect(Temp1,Y,n);                      {create dot product xY}
     IF XSq = 0 THEN BEGIN                                    {infinite slope}
        Error := 1;
        Exit;
     END;
     b := XY/XSq;                                                  {the slope}
     a := MeanY + b * MeanX;                           {correct the intercept}
END;
{this procedure checks to see if a ratio is effectively infinite, if not
 it does the addition, if so it outputs 0}

Function ComputeZero(A,B : single;VAR InfSw : Boolean):Single;
          begin
               InfSw := (B >= -1E-25);
               if Not InfSw then
                  ComputeZero := -A/B
               else
                   ComputeZero := 0;
end;

{Creates a line of ----}

FUNCTION MakeDashedLine : String;
         VAR
            TempSt : String;
        BEGIN
             FillChar(TempSt[1],79,'-');
             TempSt[0] := #79;
             MakeDashedLine := Tempst;
END;

{Centers a string assuming margins of 1,80}

FUNCTION MakeCenteredString(St : String):String;
         VAR
            TempSt : String;
            Start  : Byte;
            Ct,Ctr : Byte;
         BEGIN
              IF St[0] > #78 THEN
                 St[0] := #78;
              Start := (80 - Byte(St[0])) SHR 1;
              FillChar(TempSt[1],80,#32);
              TempSt[0] := Char(Start + Pred(Byte(St[0])));
              Ctr := 0;
              FOR Ct := Start TO (Pred(Start) + Byte(St[0])) DO BEGIN
                  Inc(Ctr);
                  TempSt[Ct] := St[Ctr];
              END;
              MakeCenteredString := TempSt;
END;

{The output procedure.  Outputs summary statistics to a file}

Procedure SummaryStats(VAR OutF                             : Text;
                           MeanDistDeath,MeanDistPair,
                           PopSz,Extinctions,
                           MeanExtinctTime,ProjExtinctTime  : Single;
                           MinPop                           : Word;
                           PrinterSw                        : Boolean);
          begin
               TextAttr := 15;
               clrscr;
               WriteLn(OutF,MakeDashedLine);
               WriteLn(OutF,MakeCenteredString('SUMMARY STATISTICS'));
               WriteLn(OutF);
               WriteLn(Outf,'The mean distance traveled before death   : ',
                       MeanDistDeath:10:2);
               WriteLn(Outf,'The mean distance traveled before pairing : ',
                       MeanDistPair:10:2);
               WriteLn(OutF);
               WriteLn(Outf,'Mean population size : ',PopSz:10:2);
               WriteLn(Outf);
               WriteLn(Outf,'Percent extinctions       : ',
                       Extinctions * 100:10:2);
               WriteLn(OutF,'Mean time to extinction   : ',
                       MeanExtinctTime:10:2);
               WriteLn(Outf,'Projected extinction time : ',
                       ProjExtinctTime:10:2);
               WriteLn(Outf,'Minimum population level  : ',
                       MinPop:8);
               WriteLn(OutF,MakeDashedLine);
               if PrinterSw then
                  Write(OutF,#12);
end;

{Calculation procedure for summary statistics}

Procedure ProduceSummaryStats(    Runs,Years           : Word;
                              VAR TotalPop             : RealDataArray;
                                  ExtinctionTime       : Single;
                                  Extinctions          : Word;
                                  MinPop               : Word;
                                  MeanDistDeath,
                                  MeanDistPair         : Single;
                              VAR OutFil               : Text;
                                  PrinterOn            : Boolean);
          VAR
             Scratch                     : RealDataArray;
             MeanExtTime,Slope,Intercept,
             ProbExtTime,MeanPop,
             MeanDeathAge                : Single;
             Err                         : Byte;
             PosSlope                    : Boolean;
             Ct                          : Word;
          begin
               For Ct := 1 to Years do
                   Scratch[Ct] := Ct;
               LeastSquare(Years,Scratch,TotalPop,Intercept,Slope,err);
               if Err = 1 then
                  ProbExtTime := -9
               else begin
                   ProbExtTime := ComputeZero(Intercept,Slope,PosSlope);
                   if PosSlope then
                      ProbExtTime := -9;
               end;
               MeanPop       := Mean(Years,TotalPop);
               MeanDistDeath := MeanDistDeath;
               MeanDistPair  := MeanDistPair;
               SummaryStats(OutFil,MeanDistDeath,MeanDistPair,
                            MeanPop,Extinctions/Runs,
                            ExtinctionTime,ProbExtTime,MinPop,PrinterOn);
end;
Procedure CreateSumStatsFile(MeanTimeExt,MeanDeathDist,
                             MeanPairDist                         : Single;
                             MinimumPop,ExtNo                     : Word;
                             FileName                             : String);
          VAR
             Stats : SumStatRec;
             OutF  : File of SumStatRec;
          begin
               Stats.MTExtinct := MeanTimeExt;
               Stats.AvgDeath  := MeanDeathDist;
               Stats.AvgPair   := MeanPairDist;
               Stats.MinPop    := MinimumPop;
               Stats.ExtNo     := ExtNo;
               Assign(OutF,ModifyFileName(FileName,'SSX'));
               ReWrite(OutF);
               if IOResult <> 0 then
                  FatalErrors(5);
               Write(OutF,Stats);
               Close(OutF);
end;
Procedure ReadSumStatsFile(VAR MeanTimeExt,MeanDeathDist,
                               MeanPairDist                     : Single;
                           VAR MinimumPop,ExtNo                 : Word;
                               FileName                         : String);
          VAR
             Stats : SumStatRec;
             OutF  : File of SumStatRec;
          begin
               Assign(OutF,ModifyFileName(FileName,'SSX'));
               ReSet(OutF);
               if IOResult <> 0 then
                  FatalErrors(5);
               Read(OutF,Stats);
               Close(OutF);
               MeanTimeExt   := Stats.MTExtinct;
               MeanDeathDist := Stats.AvgDeath;
               MeanPairdist  := Stats.AvgPair;
               MinimumPop    := Stats.MinPop;
               ExtNo         := Stats.ExtNo;
end;
PROCEDURE OutputRules(FileName : String;VAR OutF2 : Text);
          VAR
             OutF : Text;
             St   : String;
             LastCh : Byte;
          BEGIN
               LastCh := LastCharPos('.',FileName);
               IF LastCh <> 0 THEN
                  FileName[0] := Char(Pred(LastCh));
               FileName := FileName + '.QZX';
               Assign(OutF,FileName);
               ReSet(OutF);
               IF IOResult <> 0 THEN
                  FatalErrors(6);
               WHILE NOT Eof(OutF) DO BEGIN
                     ReadLn(OutF,St);
                     WriteLn(OutF2,St);
               END;
               Close(OutF);
END;
FUNCTION MakeCenteredString2(St : String):String;
         VAR
            TempSt : String;
            Start  : Byte;
            Ct,Ctr : Byte;
         BEGIN
              IF St[0] > #78 THEN
                 St[0] := #78;
              Start := (80 - Byte(St[0])) SHR 1;
              FillChar(TempSt[1],80,#32);
              TempSt[0] := Char(Start + Pred(Byte(St[0])));
              Ctr := 0;
              FOR Ct := Start TO (Pred(Start) + Byte(St[0])) DO BEGIN
                  Inc(Ctr);
                  TempSt[Ct] := St[Ctr];
              END;
              MakeCenteredString2 := TempSt;
END;
PROCEDURE WriteNameAndComment(Name,Comment : String;VAR OutF : Text);
          VAR
             Ct : Word;
          BEGIN
               FOR Ct := 1 TO 20 DO
                   WriteLn(OutF);
               WriteLn(OutF,MakeCenteredString2(Name));
               WriteLn(OutF);
               WriteLn(OutF,Comment);

END;

FUNCTION MakeStringfield(St : String;FieldWidth : Byte):String;
         VAR
           TempSt : String;
            Ct,Ctr : Byte;
         BEGIN
              FillChar(TempSt[1],FieldWidth,#32);
              TempSt[0] := Char(FieldWidth);
              IF Byte(St[0]) > FieldWidth THEN
                 St[0] := Char(FieldWidth);
              Ctr := Succ(Byte(St[0]));
              FOR Ct := FieldWidth DOWNTO (FieldWidth - Pred(Byte(St[0]))) DO
                  BEGIN
                  Dec(Ctr);
                  TempSt[Ct] := St[Ctr];
              END;
              MakeStringField := TempSt;

END;
FUNCTION MakeRealField(Mean,Std : Single;Field : Word):String;

         VAR
            TempSt,TempSt2 : String;
            Decimals       : Word;
         BEGIN
              IF Mean >= 1000 THEN
                 Decimals := 0
              ELSE IF Mean >= 100 THEN
                 Decimals := 1
              ELSE IF Mean < 1 THEN
                   Decimals := 3
              ELSE
                  Decimals := 2;
              Str(Mean:5:Decimals,TempSt);
              IF Std >= 1000 THEN
                 Decimals := 0
              ELSE IF Std >= 100 THEN
                 Decimals := 1
              ELSE IF Std < 1 THEN
                   Decimals := 3
              ELSE
                  Decimals := 2;
              Str(Std:5:Decimals,TempSt2);
              MakeRealField := MakeStringField(TempSt +' '+TempSt2,Field);
END;



PROCEDURE StartDataType(VAR Outf  : Text;
                            St    : String;
                            Arr   : TitleArr;
                            N     : Byte;
                            Field : Byte);
          VAR
             Ct : Byte;
          BEGIN
          WriteLn(OutF,MakeDashedLine);
          Write(OutF,MakeStringField(St,Field));
          FOR Ct := 1 TO N DO
              Write(OutF,MakeStringField(Arr[Ct],Field));
          WriteLn(OutF,'');
          WriteLn(OutF,MakeDashedLine);
END;


{PROCEDURE FullData(VAR OutF : Text;YrCtr : Word);
          VAR
             Ct,i  : Word;
             Field : Word;
          BEGIN
               Field := 135 DIV 10;
               WriteLn(OutF,MakeCenteredString('CUMULATIVE YEARLY DATA'));
               WriteLn(OutF);
               StartDataType(Outf,'YEAR',OutTitle,LastStage + 3,Field);
               FOR Ct := 1 TO YrCtr DO BEGIN
                  Write(OutF,Ct:Field);
                  Write(OutF,MakeRealField(Pop^[Ct],PopSD^[Ct],Field));
                   Write(OutF,MakeRealField(Pair^[Ct],PairSD^[Ct],Field));
                  FOR i := 0 TO LastStage DO
                     Write(OutF,MakeRealField(OwlTotal[i]^[Ct],OwlSD[i]^[Ct],Field));
                  WriteLn(OutF);
               END;
END;}



PROCEDURE CenterText(Y,Att : Byte; St : String);
          VAR
             Posit : Word;
          BEGIN
               Posit := Pred(Y) * 160 + (80 - (Byte(St[0]) DIV 2) * 2);
               VarFWrite(Posit,Att,St);
END;

PROCEDURE OutputToScreen(VAR Inf : Text; FN : String);
VAR
  Outf : Text;
  Ch1, Ch2 : Char;
BEGIN

    RestoreCrtMode;
    AssignCrt(Outf);
    ReWrite(Outf);
    ReadSumStatsFile(MeanExtTime,
         MeanDeathDist,
         MeanPairDist,
         MinPop,ExtinctNo,
         FN);
    ProduceSummaryStats(Header.Reps,Header.Years,Pop^,
         MeanExtTime,
         ExtinctNo,MinPop,
         MeanDeathDist,
         MeanPairDist,OutF,
         False);
    CenterText(25,RevVid,'Any key to continue');
    GetKeys(Ch1,Ch2);
    InitializeGraphics;
    Close(Outf);
END;
begin
end.





